import math

class Point:
    def __init__(self, x, y):
        self.x = x
        self.y = y

    def __add__(self, other):
        return self.__class__(self.x + other.x, self.y + other.y)

    def __sub__(self, other):
        return self.__class__(self.x - other.x, self.y - other.y)

    def __neg__(self):
        return self.__class__(-self.x, -self.y)

    def __div__(self, denom):
        return self.__class__(self.x / denom, self.y / denom)

    def __mul__(self, denom):
        return self.__class__(self.x * denom, self.y * denom)

    def __cmp__(self, other):
        return cmp(self.x, other.x) or cmp(self.y, other.y)

    def angle_to(self, other):
        dx = other.x - self.x
        dy = other.y - self.y
        if dx == 0:
            angle = -math.pi / 2 if dy < 0 else math.pi / 2
        else:
            angle = math.atan(dy/dx)
            if dx < 0:
                angle = -math.pi + angle

        angle += math.pi * 2
        while angle >= math.pi:
            angle -= math.pi * 2
        return angle

    @classmethod
    def rt(cls, r, theta):
        return cls(math.cos(theta) * r, math.sin(theta) * r)

    def compose(self):
        return self.__class__(self.x, -self.y)

class Line:
    def __init__(self, a, b):
        self.a = a
        self.b = b

    @staticmethod
    def _det(a, b):
        return a.x * b.y - a.y * b.x

    def intersects(self, other):
        xd = Point(self.a.x - self.b.x, other.a.x - other.b.x)
        yd = Point(self.a.y - self.b.y, other.a.y - other.b.y)

        div = self._det(xd, yd)
        if abs(div) < 1e-10:
           raise ValueError('Lines do not intersect')

        d = Point(self._det(self.a, self.b), self._det(other.a, other.b))

        return self.a.__class__(self._det(d, xd), self._det(d, yd)) / div

class Quad:
    def __init__(self, a, b, c, d):
        self.corners = a, b, c, d

    @property
    def a(self):
        return self.corners[0]

    @property
    def b(self):
        return self.corners[1]

    @property
    def c(self):
        return self.corners[2]

    @property
    def d(self):
        return self.corners[3]

    def __sub__(self, point):
        return self + (-point)

    def __add__(self, point):
        return self.__class__(*map(point.__add__, self.corners))

    @property
    def center(self):
        return sum(self.corners, Point(0,0)) / 4

    @classmethod
    def from_segment(cls, segment, width):
        angle = segment.a.angle_to(segment.b)
        return cls(segment.a + Point.rt(width / 2, angle - math.pi / 2), segment.a + Point.rt(width / 2, angle + math.pi / 2),
                   segment.b + Point.rt(width / 2, angle + math.pi / 2), segment.b + Point.rt(width / 2, angle - math.pi / 2))

    def connect_to(self, other):
        if self.corners[2] == other.corners[1]:
            return

        sup = Line(self.b, self.c)
        sdn = Line(self.d, self.a)
        oup = Line(other.b, other.c)
        odn = Line(other.d, other.a)

        cup = sup.intersects(oup)
        cdn = sdn.intersects(odn)

        self.corners = self.a, self.b, cup, cdn
        other.corners = cdn, cup, other.c, other.d

class Rectangle:
    point_class = Point

    def __init__(self, top, right, bottom, left):
        self.top = top
        self.right = right
        self.bottom = bottom
        self.left = left

    @classmethod
    def containing(cls, *points):
        return cls(min([p.y for p in points]),
                   max([p.x for p in points]),
                   max([p.y for p in points]),
                   min([p.x for p in points]))

    def enlarged(self, w, h = None):
        if h is None:
            h = w
        return Rectangle(self.top - h, self.right + w, self.bottom + h, self.left - h)

    def __or__(self, other):
        return Rectangle(min(self.bottom, other.bottom, self.top, other.top),
                         max(self.right, other.right, self.left, other.left),
                         max(self.bottom, other.bottom, self.top, other.top),
                         min(self.right, other.right, self.left, other.left))

    def __ior__(self, other):
        self.top = min(self.bottom, other.bottom, self.top, other.top)
        self.right = max(self.right, other.right, self.left, other.left)
        self.bottom = max(self.bottom, other.bottom, self.top, other.top)
        self.left = min(self.right, other.right, self.left, other.left)
        return self

    @property
    def tc(self):
        return self.point_class((self.left + self.right) / 2, self.top)

    @property
    def bc(self):
        return self.point_class((self.left + self.right) / 2, self.bottom)

    @property
    def lc(self):
        return self.point_class(self.left, (self.top + self.bottom) / 2)

    @property
    def rc(self):
        return self.point_class(self.right, (self.top + self.bottom) / 2)

    @property
    def center(self):
        return self.point_class((self.left + self.right) / 2, (self.top + self.bottom) / 2)

    @property
    def tl(self):
        return self.point_class(self.left, self.top)

    @property
    def tr(self):
        return self.point_class(self.right, self.top)

    @property
    def bl(self):
        return self.point_class(self.left, self.bottom)

    @property
    def br(self):
        return self.point_class(self.right, self.bottom)
